@@ -35,7 +35,7 @@ if os.name == "nt":
     except ImportError:
         import winreg
 
-hererocks_version = "Hererocks 0.21.0"
+hererocks_version = "Hererocks 0.23.0"
 __all__ = ["main"]
 
 opts = None
@@ -94,9 +94,9 @@ activation_script_templates = {
         fi
     """,
     "activate_posix": """
-        s=$(command -V deactivate_lua)
+        s=$(command -V deactivate_lua 2>&1)
         if [ $? -eq 0 ]; then
-            if [ "${s##*function}" = '' ]; then
+            if [ "${s##*function*}" = '' ]; then
                 deactivate_lua
             fi;
         fi;
@@ -905,19 +905,18 @@ class RioLua(Lua):
     versions = [
         "5.1", "5.1.1", "5.1.2", "5.1.3", "5.1.4", "5.1.5",
         "5.2.0", "5.2.1", "5.2.2", "5.2.3", "5.2.4",
-        "5.3.0", "5.3.1", "5.3.2", "5.3.3", "5.3.4", "5.3.5",
-        "5.4.0", "5.4.0-rc1"
+        "5.3.0", "5.3.1", "5.3.2", "5.3.3", "5.3.4", "5.3.5", "5.3.6",
+        "5.4.0", "5.4.1", "5.4.2"
     ]
     translations = {
-        "5": "5.3.5",
+        "5": "5.4.1",
         "5.1": "5.1.5",
         "5.1.0": "5.1",
         "5.2": "5.2.4",
-        "5.3": "5.3.5",
-        "5.4": "5.4.0-rc1",
-        "5.4.0": "5.4.0-rc1",
-        "^": "5.3.5",
-        "latest": "5.3.5"
+        "5.3": "5.3.6",
+        "5.4": "5.4.2",
+        "^": "5.4.2",
+        "latest": "5.4.2"
     }
     checksums = {
         "lua-5.1.tar.gz"        : "7f5bb9061eb3b9ba1e406a5aa68001a66cb82bac95748839dc02dd10048472c1",
@@ -937,7 +936,10 @@ class RioLua(Lua):
         "lua-5.3.3.tar.gz"      : "5113c06884f7de453ce57702abaac1d618307f33f6789fa870e87a59d772aca2",
         "lua-5.3.4.tar.gz"      : "f681aa518233bc407e23acf0f5887c884f17436f000d453b2491a9f11a52400c",
         "lua-5.3.5.tar.gz"      : "0c2eed3f960446e1a3e4b9a1ca2f3ff893b6ce41942cf54d5dd59ab4b3b058ac",
-        "lua-5.4.0-rc1.tar.gz"  : "0a4fb4cb9281d924799650a768e61723fac3f0329bca39e51b90d1acc4228e71",
+        "lua-5.3.6.tar.gz"      : "fc5fd69bb8736323f026672b1b7235da613d7177e72558893a0bdcd320466d60",
+        "lua-5.4.0.tar.gz"      : "eac0836eb7219e421a96b7ee3692b93f0629e4cdb0c788432e3d10ce9ed47e28",
+        "lua-5.4.1.tar.gz"      : "4ba786c3705eb9db6567af29c91a01b81f1c0ac3124fdbf6cd94bdd9e53cca7d",
+        "lua-5.4.2.tar.gz"      : "11570d97e9d7303c0a59567ed1ac7c648340cd0db10d5fd594c09223ef2f524f",
     }
     all_patches = {
         "When loading a file, Lua may call the reader function again after it returned end of input": """
@@ -1226,6 +1228,141 @@ class RioLua(Lua):
                luaC_upvdeccount(L, *up1);
                *up1 = *up2;
                (*up1)->refcount++;
+        """,
+        "Old finalized object may not be visited by GC": """
+            lgc.c:
+            @@ -1140,7 +1140,7 @@ static void finishgencycle (lua_State *L, global_State *g) {
+             static void youngcollection (lua_State *L, global_State *g) {
+               GCObject **psurvival;  /* to point to first non-dead survival object */
+               lua_assert(g->gcstate == GCSpropagate);
+            -  markold(g, g->survival, g->reallyold);
+            +  markold(g, g->allgc, g->reallyold);
+               markold(g, g->finobj, g->finobjrold);
+               atomic(L);
+        """,
+        "Computation of stack limit when entering a coroutine is wrong": """
+            ldo.c:
+            @@ -674,7 +674,7 @@ LUA_API int lua_resume (lua_State *L, lua_State *from, int nargs,
+               if (from == NULL)
+                 L->nCcalls = CSTACKTHREAD;
+               else  /* correct 'nCcalls' for this thread */
+            -    L->nCcalls = getCcalls(from) + from->nci - L->nci - CSTACKCF;
+            +    L->nCcalls = getCcalls(from) - L->nci - CSTACKCF;
+               if (L->nCcalls <= CSTACKERR)
+                 return resume_error(L, "C stack overflow", nargs);
+               luai_userstateresume(L, nargs);
+        """,
+        "An emergency collection when handling an error while loading the upvalues of a function can cause a segfault": """
+            lundump.c:
+            @@ -205,8 +205,9 @@ static void loadUpvalues (LoadState *S, Proto *f) {
+               n = loadInt(S);
+               f->upvalues = luaM_newvectorchecked(S->L, n, Upvaldesc);
+               f->sizeupvalues = n;
+            -  for (i = 0; i < n; i++) {
+            +  for (i = 0; i < n; i++)
+                 f->upvalues[i].name = NULL;
+            +  for (i = 0; i < n; i++) {
+                 f->upvalues[i].instack = loadByte(S);
+                 f->upvalues[i].idx = loadByte(S);
+                 f->upvalues[i].kind = loadByte(S);
+        """,
+        "'checkstackp' can run a GC step and destroy a preallocated CallInfo": """
+            ldo.c:
+            @@ -466,13 +466,13 @@ void luaD_call (lua_State *L, StkId func, int nresults) {
+                   f = fvalue(s2v(func));
+                  Cfunc: {
+                   int n;  /* number of returns */
+            -      CallInfo *ci = next_ci(L);
+            +      CallInfo *ci;
+                   checkstackp(L, LUA_MINSTACK, func);  /* ensure minimum stack size */
+            +      L->ci = ci = next_ci(L);
+                   ci->nresults = nresults;
+                   ci->callstatus = CIST_C;
+                   ci->top = L->top + LUA_MINSTACK;
+                   ci->func = func;
+            -      L->ci = ci;
+                   lua_assert(ci->top <= L->stack_last);
+                   if (L->hookmask & LUA_MASKCALL) {
+                     int narg = cast_int(L->top - func) - 1;
+            @@ -486,18 +486,18 @@ void luaD_call (lua_State *L, StkId func, int nresults) {
+                   break;
+                 }
+                 case LUA_VLCL: {  /* Lua function */
+            -      CallInfo *ci = next_ci(L);
+            +      CallInfo *ci;
+                   Proto *p = clLvalue(s2v(func))->p;
+                   int narg = cast_int(L->top - func) - 1;  /* number of real arguments */
+                   int nfixparams = p->numparams;
+                   int fsize = p->maxstacksize;  /* frame size */
+                   checkstackp(L, fsize, func);
+            +      L->ci = ci = next_ci(L);
+                   ci->nresults = nresults;
+                   ci->u.l.savedpc = p->code;  /* starting point */
+                   ci->callstatus = 0;
+                   ci->top = func + 1 + fsize;
+                   ci->func = func;
+            -      L->ci = ci;
+                   for (; narg < nfixparams; narg++)
+                     setnilvalue(s2v(L->top++));  /* complete missing arguments */
+                   lua_assert(ci->top <= L->stack_last);
+        """,
+        "GC after resizing stack can shrink it again": """
+            ldo.h:
+            @@ -44,7 +44,7 @@
+
+             /* macro to check stack size and GC */
+             #define checkstackGC(L,fsize)  \\
+            -	luaD_checkstackaux(L, (fsize), (void)0, luaC_checkGC(L))
+            +	luaD_checkstackaux(L, (fsize), luaC_checkGC(L), (void)0)
+
+
+             /* type of protected functions, to be ran by 'runprotected' */
+        """,
+        "Errors in finalizers need a valid 'pc' to produce an error message": """
+            lvm.c:
+            @@ -1104,7 +1104,7 @@ void luaV_finishOp (lua_State *L) {
+
+
+             #define checkGC(L,c)  \\
+            -	{ luaC_condGC(L, L->top = (c),  /* limit of live values */ \\
+            +	{ luaC_condGC(L, (savepc(L), L->top = (c)), \\
+                                      updatetrap(ci)); \\
+                        luai_threadyield(L); }
+
+            @@ -1792,8 +1792,7 @@ void luaV_execute (lua_State *L, CallInfo *ci) {
+                     vmbreak;
+                   }
+                   vmcase(OP_VARARGPREP) {
+            -        luaT_adjustvarargs(L, GETARG_A(i), ci, cl->p);
+            -        updatetrap(ci);
+            +        ProtectNT(luaT_adjustvarargs(L, GETARG_A(i), ci, cl->p));
+                     if (trap) {
+                       luaD_hookcall(L, ci);
+                       L->oldpc = pc + 1;  /* next opcode will be seen as a "new" line */
+        """,
+        "'popen' can crash if called with an invalid mode": """
+            liolib.c:
+            @@ -279,6 +279,8 @@ static int io_popen (lua_State *L) {
+               const char *filename = luaL_checkstring(L, 1);
+               const char *mode = luaL_optstring(L, 2, "r");
+               LStream *p = newprefile(L);
+            +  luaL_argcheck(L, ((mode[0] == 'r' || mode[0] == 'w') && mode[1] == '\0'),
+            +                   2, "invalid mode");
+               p->f = l_popen(L, filename, mode);
+               p->closef = &io_pclose;
+               return (p->f == NULL) ? luaL_fileresult(L, 0, filename) : 1;
+        """,
+        "Field 'L->oldpc' is not always updated when returning to a function": """
+            lgc.c:
+            @@ -856,6 +856,8 @@ static void GCTM (lua_State *L) {
+                 if (unlikely(status != LUA_OK)) {  /* error while running __gc? */
+                   luaE_warnerror(L, "__gc metamethod");
+                   L->top--;  /* pops error object */
+            +      if (isLua(L->ci))
+            +        L->oldpc = L->ci->u.l.savedpc;  /* update 'oldpc' */
+                 }
+               }
+             }
         """
     }
     patches_per_version = {
@@ -1257,7 +1394,19 @@ class RioLua(Lua):
             "5": [
                 "Joining an upvalue with itself can cause a use-after-free crash"
             ]
-        }
+        },
+        "5.4": {
+            "0": [
+                "Old finalized object may not be visited by GC",
+                "Computation of stack limit when entering a coroutine is wrong",
+                "An emergency collection when handling an error while loading the upvalues of a function can cause a segfault",
+                "'checkstackp' can run a GC step and destroy a preallocated CallInfo",
+                "GC after resizing stack can shrink it again",
+                "Errors in finalizers need a valid 'pc' to produce an error message",
+                "'popen' can crash if called with an invalid mode",
+                "Field 'L->oldpc' is not always updated when returning to a function"
+            ]
+        },
     }
 
     def __init__(self, version):
@@ -1749,7 +1898,9 @@ class LuaRocks(Program):
         "3.0.0", "3.0.1", "3.0.2", "3.0.3", "3.0.4",
         "3.1.0", "3.1.1", "3.1.2", "3.1.3",
         "3.2.0", "3.2.1",
-        "3.3.0", "3.3.1"
+        "3.3.0", "3.3.1",
+        "3.4.0",
+        "3.5.0",
     ]
     translations = {
         "2": "2.4.4",
@@ -1758,13 +1909,15 @@ class LuaRocks(Program):
         "2.2": "2.2.2",
         "2.3": "2.3.0",
         "2.4": "2.4.4",
-        "3": "3.2.1",
+        "3": "3.4.0",
         "3.0": "3.0.4",
         "3.1": "3.1.3",
         "3.2": "3.2.1",
         "3.3": "3.3.1",
-        "^": "3.3.1",
-        "latest": "3.3.1"
+        "3.4": "3.4.0",
+        "3.5": "3.5.0",
+        "^": "3.5.0",
+        "latest": "3.5.0"
     }
     checksums = {
         "luarocks-2.0.10.tar.gz"   : "11731dfe6e210a962cb2a857b8b2f14a9ab1043e13af09a1b9455b486401b46e",
@@ -1829,6 +1982,10 @@ class LuaRocks(Program):
         "luarocks-3.3.0-win32.zip" : "0c60d7e187b8547cfa4f44372a7d8cc50bafa48cdcea6eb6889094e4681718b4",
         "luarocks-3.3.1.tar.gz"    : "eb20cd9814df05535d9aae98da532217c590fc07d48d90ca237e2a7cdcf284fe",
         "luarocks-3.3.1-win32.zip" : "5ba3f6034e26e0bf93145354dca6aadb0bd7ad85c85e603a323787273db508bb",
+        "luarocks-3.4.0.tar.gz"    : "62ce5826f0eeeb760d884ea8330cd1552b5d432138b8bade0fa72f35badd02d0",
+        "luarocks-3.4.0-win32.zip" : "cd2b26d34f36cfbf69e3d80a8d25c46529e367f0866c41ac96049081a5653fc0",
+        "luarocks-3.5.0.tar.gz"    : "701d0cc0c7e97cc2cf2c2f4068fce45e52a8854f5dc6c9e49e2014202eec9a4f",
+        "luarocks-3.5.0-win32.zip" : "69a225f339a96cdbfe8eb5fcfb108c2eef270dab172bc09e78190a8a0932ffc8",
     }
 
     def get_download_name(self):
@@ -2227,8 +2384,8 @@ def main(argv=None):
     parser.add_argument(
         "-l", "--lua", help="Version of standard PUC-Rio Lua to install. "
         "Version can be specified as a version number, e.g. 5.2 or 5.3.1. "
-        "Versions 5.1.0 - 5.3.5 and 5.4.0-beta are supported. "
-        "'latest' and '^' are aliases for 5.3.5. "
+        "Versions 5.1.0 - 5.4.2 are supported. "
+        "'latest' and '^' are aliases for 5.4.2. "
         "If the argument contains '@', sources will be downloaded "
         "from a git repo using URI before '@' and using part after '@' as git reference "
         "to checkout, 'master' by default. "
@@ -2257,8 +2414,8 @@ def main(argv=None):
     parser.add_argument(
         "-r", "--luarocks", help="Version of LuaRocks to install. "
         "Version can be specified in the same way as for standard Lua. "
-        "Versions 2.0.8 - 3.3.1 are supported. "
-        "'latest' and '^' are aliases for 3.3.1. "
+        "Versions 2.0.8 - 3.5.0 are supported. "
+        "'latest' and '^' are aliases for 3.5.0. "
         "Default git repo is https://github.com/luarocks/luarocks. "
         "Note that Lua 5.2 is not supported in LuaRocks 2.0.8, "
         "Lua 5.3 is supported only since LuaRocks 2.2.0, Lua 5.4 is supported only since "
